# Needs the data type of a position
from position import pos, get_x, get_y

def ship(p1,p2):
    """Constructs a ship instance.

    p1 --  bow position (front part)
    p2 --  stern position (rear part)
    """
    return [p1, p2]

def ship_pos(s, p):
    """Checks if ship in position.

    s -- ship
    p -- position
    """
    if get_x(s[0]) == get_x(s[1]):
        return get_x(s[0]) == get_x(p) and \
            (get_y(s[0]) <= get_y(p) <= get_y(s[1]) or get_y(s[1]) <= get_y(p) <= get_y(s[0]))
    else:
        return get_y(s[0]) == get_y(p) and \
                (get_x(s[0]) <= get_x(p) <= get_x(s[1]) or get_x(s[1]) <= get_x(p) <= get_x(s[0]))
                 
def board(ship_list):
    """Construct battleship game board.

    ship_list -- list of ships to attach on the board
    """
    def board_func(pos):
        i = 0
        while i < len(ship_list):
            if ship_pos(ship_list[i], pos):
                return True
            i += 1
        return False
    
    return board_func

def is_occupied(board, pos):
     """Checks if a position is occupied by a ship.

     board -- game board
     pos -- position to check

     Return value if True if position in pos is occupied by a ship on the board.
     """
     return board(pos)

def print_board(board):
    """Print battleship board.

    board -- game board to print

    Returns None.
    """
    y = 1
    while y <= 10:
        y_str, x = '', 1
        while x <= 10:
            y_str = y_str + ('1' if is_occupied(board, pos(x, y)) else '0')
            x += 1
        y += 1
        print(y_str)

def game_over(board):
    """Checks if all ships are sunk."""
    y = 1
    while y <= 10:
        x = 0
        while x <= 10:
            if is_occupied(board, pos(x, y)):
                return False
            x += 1
        y += 1
    return True

def fire(board_func, pos):
    """Fire.
    
    board_func -- game board
    pos -- position to fire
    """
    def new_board_func(p):
        if p == pos:
            return False
        else:
            return board_func(p)
    return new_board_func
